package com.rm.dnndash;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.*;
import com.rm.dnndash.data.HttpRequestHelper;
import org.apache.http.client.ResponseHandler;

public class Main extends Activity {

    private static final String CLASSTAG = Main.class.getSimpleName();

    private final String UriSchemeHttps = "https";
    private final String UriSchemeHttp = "http";
    private final String SchemeDelimiter = "://";

    private EditText httpAlias;
    private EditText user;
    private EditText pass;
    private Button uiRefresh;
    private Button uiControlsMenu;

    private ProgressDialog progressDialog;

    private String xml;

    // use a handler to update the UI (send the handler messages from other threads)
    private final Handler handler = new Handler() {

        @Override
        public void handleMessage(final Message msg) {
            progressDialog.dismiss();
            xml = msg.getData().getString("RESPONSE");
            if(xml != null)
                uiControlsMenu.setEnabled(true);
        }
    };

    @Override
    public void onCreate(final Bundle icicle) {
        super.onCreate(icicle);

        // Main view
        this.setContentView(R.layout.main);

        this.httpAlias = (EditText) findViewById(R.id.htth_url);
        this.user = (EditText) findViewById(R.id.htth_user);
        this.pass = (EditText) findViewById(R.id.htth_pass);
        this.uiRefresh = (Button) findViewById(R.id.htth_refresh_button);
        this.uiControlsMenu = (Button) findViewById(R.id.htth_controls_menu_button);
        this.uiControlsMenu.setEnabled(false);

        this.uiRefresh.setOnClickListener(new View.OnClickListener() {

            public void onClick(final View v) {

                // Build URL
                // TODO Options for Url definition
                String url = GetUrl(httpAlias.getText().toString(), "DnnDash_SimpleDashboard", "Dash", "Controls", false);

                // Do request
                performRequest(url, "POST", user.getText().toString(), pass.getText().toString());
            }
        });

        this.uiControlsMenu.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                Intent intent = new Intent(getApplicationContext(), MenuActivity.class);
                intent.putExtra(Constants.XML_KEY, xml);
                startActivity(intent);
            }
        });
    };

    private String GetUrl(String portalAlias, String modulePath, String controllerName, String callName, Boolean secure)
    {

        String scheme = "";
        if (secure)
            scheme = UriSchemeHttps + SchemeDelimiter;
        else
            scheme = UriSchemeHttp + SchemeDelimiter;

        String url = String.format("%1$s%2$s/DesktopModules/%3$s/API/%4$s/%5$s", scheme, portalAlias, modulePath, controllerName, callName);

        return url;

    }

    @Override
    public void onPause() {
        super.onPause();
        if ((this.progressDialog != null) && this.progressDialog.isShowing()) {
            this.progressDialog.dismiss();
        }
    }

    /**
     * Perform asynchronous HTTP using Apache <code>HttpClient</code> via
     * <code>HttpRequestHelper</code> and <code>ResponseHandler</code>.
     *
     * @param url
     * @param method
     * @param user
     * @param pass
     */
    private void performRequest(final String url, final String method, final String user, final String pass) {

        Log.d(Constants.LOGTAG, " " + Main.CLASSTAG + " request url - " + url);
        Log.d(Constants.LOGTAG, " " + Main.CLASSTAG + " request method - " + method);
        Log.d(Constants.LOGTAG, " " + Main.CLASSTAG + " user - " + user);
        Log.d(Constants.LOGTAG, " " + Main.CLASSTAG + " password - " + pass);

        final ResponseHandler<String> responseHandler = HttpRequestHelper.getResponseHandlerInstance(this.handler);

        this.progressDialog = ProgressDialog.show(this, "working . . .", "performing HTTP request");

        // do the HTTP dance in a separate thread (the responseHandler will fire when complete)
        new Thread() {

            @Override
            public void run() {
                HttpRequestHelper helper = new HttpRequestHelper(responseHandler);
                if (method.equals("GET")) {
                    helper.performGet(url, user, pass, null);
                } else if (method.equals("POST")) {
                    helper.performPost(HttpRequestHelper.MIME_FORM_ENCODED, url, user, pass, null);
                } else {
                    Message msg = handler.obtainMessage();
                    Bundle bundle = new Bundle();
                    bundle.putString("RESPONSE", "ERROR - see logcat");
                    msg.setData(bundle);
                    handler.sendMessage(msg);
                    Log.w(Constants.LOGTAG, " " + Main.CLASSTAG + " unknown method, nothing to do");
                }
            }
        }.start();
    }
}
