package com.rm.dnndash.data;

import android.util.Log;
import org.w3c.dom.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: ROB
 * Date: 7/10/12
 * Time: 6:34 PM
 * To change this template use File | Settings | File Templates.
 */
public class DashParser {

    private static final String CLASSTAG = DashParser.class.getSimpleName();
    static final String DASH_CONTROL = "dashControl"; // Element
    static final String DASH_CONTROL_NAME = "name"; // Attribute
    static Document dom;

    public static ControlXmlList parseDashControls(String xml) {

        ControlXmlList dashControls = new ControlXmlList();

        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            dom = builder.parse(new ByteArrayInputStream(xml.getBytes("UTF-8")));
            NodeList nodeList = dom.getElementsByTagName(DASH_CONTROL);
            for(int i = 0; i < nodeList.getLength(); i++) {
                Node node = nodeList.item(i);
                if(node.getAttributes()!= null) {
                    Node attr = node.getAttributes().getNamedItem(DASH_CONTROL_NAME);
                    if(attr != null) {
                        ControlXml ctrlXml = new ControlXml();
                        ctrlXml.name = attr.getNodeValue();
                        //ctrlXml.nodes = attr.getChildNodes(); // TODO with Serializable

                        TransformerFactory transformerFactory = TransformerFactory.newInstance();
                        Transformer transformer = transformerFactory.newTransformer();
                        Source source = new DOMSource(node);
                        StringWriter writer = new StringWriter();
                        StreamResult result = new StreamResult(writer);
                        //Result result = new StreamResult(System.out);

                        transformer.transform(source, result);
                        Log.d(CLASSTAG, "Result: " + writer.toString());
                        ctrlXml.xml = writer.toString(); // inner xml for control

                        dashControls.add(ctrlXml);
                    }
                }
            }
        }
        catch (Exception ex)
        {
            Log.d(CLASSTAG, ex.getMessage());
        }

        return dashControls;
    }

    public static List<Setting> parseControlSettings(String xml, String controlName){

        List<Setting> settings = new ArrayList<Setting>();

        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            dom = builder.parse(new ByteArrayInputStream(xml.getBytes("UTF-8")));
            NodeList nodeList = dom.getChildNodes(); // ?
            Log.d(CLASSTAG, "Children count: " + nodeList.getLength());

            if(controlName.equals("Web Server") ||
               controlName.equals("Host")) { // TODO simplify

                for(int i = 0; i < nodeList.getLength(); i++) {
                    Node node = nodeList.item(i);
                    if(node.hasChildNodes()) {
                        NodeList children1 = node.getChildNodes();
                        Log.d(CLASSTAG, "Children1 count: " + nodeList.getLength());

                        for(int j = 0; j < children1.getLength(); j++) {

                            Node child1 = children1.item(j);

                            if(child1.getNodeName().equals("server") ||
                                child1.getNodeName().equals("host")) {

                                NodeList children2 = child1.getChildNodes();
                                Log.d(CLASSTAG, "Children2 count: " + children2.getLength());

                                for(int k = 0; k < children2.getLength(); k++) {
                                    Node child2 = children2.item(k);
                                    Setting setting = new Setting();
                                    setting.key = child2.getNodeName();
                                    setting.value = child2.getTextContent();
                                    settings.add(setting);
                                } // k
                            }
                        } // j
                    }
                } // i
            } else {

                for(int i = 0; i < nodeList.getLength(); i++) {
                    Node node = nodeList.item(i);
                    if(node.hasChildNodes()) {
                        NodeList children1 = node.getChildNodes();
                        Log.d(CLASSTAG, "Children1 count: " + nodeList.getLength());

                        for(int j = 0; j < children1.getLength(); j++) {

                            Node child1 = children1.item(j);

                            if(child1.getNodeName().equals("portalsSocialData") ||
                                child1.getNodeName().equals("portals") ||
                                child1.getNodeName().equals("installedModules") ||
                                child1.getNodeName().equals("installedSkins")) {

                                NodeList children2 = child1.getChildNodes();
                                Log.d(CLASSTAG, "Children2 count: " + children2.getLength());

                                for(int k = 0; k < children2.getLength(); k++) {

                                    Node child2 = children2.item(k);

                                    if(child2.hasChildNodes()) {
                                        NodeList children3 = child2.getChildNodes();
                                        Log.d(CLASSTAG, "Children3 count: " + children3.getLength());

                                        for(int l = 0; l < children3.getLength(); l++) {
                                            Node child3 = children3.item(l);
                                            Setting setting = new Setting();
                                            setting.key = child3.getNodeName();
                                            setting.value = child3.getTextContent();
                                            settings.add(setting);
                                        } // l
                                    }
                                } // k
                            }
                        } // j
                    }
                } // i
            }
        }
        catch (Exception ex)
        {
            Log.d(CLASSTAG, ex.getMessage());
        }

        return settings;

    }
}
