﻿using System;
using System.Net;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;

namespace DnnDash.SimpleDashboard
{
    internal static class JsonSerializer
    {
        /// <summary>
        /// Serializes an object to a UTF-8 encoded JSON string.
        /// </summary>
        /// <param name="obj">object to serialize</param>
        /// <returns>JSON string result</returns>
        internal static string Serialize(object obj)
        {
            // Serialize to a memory stream....
            MemoryStream ms = new MemoryStream();

            // Serialize to memory stream with DataContractJsonSerializer
            DataContractJsonSerializer s = new DataContractJsonSerializer(obj.GetType());
            s.WriteObject(ms, obj);
            byte[] json = ms.ToArray();
            ms.Close();

            // return utf8 encoded json string
            return Encoding.UTF8.GetString(json, 0, json.Length);
        }

        /// <summary>
        /// Deserializes an object from a UTF-8 encoded JSON string.
        /// </summary>
        /// <typeparam name="T">type of object to deserialize as</typeparam>
        /// <param name="json">UTF-8 encoded JSON string</param>
        /// <returns>deserialized object</returns>
        internal static T Deserialize<T>(string json)
        {
            T result = default(T);

            // load json into memorystream and deserialize
            MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(json));
            DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(T));
            result = (T)s.ReadObject(ms);
            ms.Close();

            return result;
        }
    }
    [DataContract]
    internal class SimpleRequest
    {
        [DataMember]
        public object data { get; set; }
    }
    [DataContract]
    internal class SimpleResult
    {
        [DataMember]
        public string Result { get; set; }
        [DataMember]
        public string data { get; set; }
        internal SimpleResult(string _result)
        {
            Result = _result;
        }
    }
    [DataContract]
    internal class FullResult : SimpleResult
    {
        [DataMember]
        public string message { get; set; }
        internal FullResult(string _result, Exception ex)
            : base(_result)
        {
            if (ex != null)
                message = ex.Message;
            else
                message = null;
        }
    }
    [DataContract]
    internal class DebugResult : FullResult
    {
        [DataMember]
        public bool debug { get; set; }
        [DataMember]
        public string stackTrace { get; set; }
        internal DebugResult(string _result, bool isDebug, Exception ex)
            : base(_result, ex)
        {
            debug = isDebug;
            if (ex != null)
                stackTrace = ex.StackTrace;
            else
                stackTrace = null;
        }
    }
}
